@@ -1,6 +1,6 @@
 /* Miscellaneous support for test programs.
 
-Copyright 2001-2017 Free Software Foundation, Inc.
+Copyright 2001-2018 Free Software Foundation, Inc.
 Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
@@ -21,12 +21,9 @@ http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
 #ifdef HAVE_CONFIG_H
-# if HAVE_CONFIG_H
-#  include "config.h"     /* for a build within gmp */
-# endif
+# include "config.h"
 #endif
 
-#include <stdlib.h>
 #include <float.h>
 #include <errno.h>
 
@@ -34,7 +31,10 @@ http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 #include <locale.h>
 #endif
 
-#ifdef MPFR_TEST_DIVBYZERO
+#ifdef MPFR_TESTS_FPE_DIV
+# ifdef MPFR_TESTS_FPE_TRAP
+#  define _GNU_SOURCE /* for feenableexcept */
+# endif
 # include <fenv.h>
 #endif
 
@@ -57,6 +57,10 @@ http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 #include <sys/resource.h>
 #endif
 
+#if defined(HAVE_SIGNAL) || defined(HAVE_SIGACTION)
+# include <signal.h>
+#endif
+
 #include "mpfr-test.h"
 
 #ifdef MPFR_FPU_PREC
@@ -98,6 +102,20 @@ set_fpu_prec (void)
 
 #endif
 
+char             mpfr_rands_initialized = 0;
+gmp_randstate_t  mpfr_rands;
+
+char *locale = NULL;
+
+/* Programs that test GMP's mp_set_memory_functions() need to set
+   tests_memory_disabled = 2 before calling tests_start_mpfr(). */
+#ifdef MPFR_USE_MINI_GMP
+/* disable since mini-gmp does not keep track of old_size in realloc/free */
+int tests_memory_disabled = 1;
+#else
+int tests_memory_disabled = 0;
+#endif
+
 static mpfr_exp_t default_emin, default_emax;
 
 static void tests_rand_start (void);
@@ -119,7 +137,8 @@ void (*dummy_func)(mpfr_srcptr) = mpfr_dump;
    of "make check") but a different library that is already installed,
    i.e. any test result would be meaningless; in such a case, we exit
    immediately with an error (exit status = 1).
-   Return value: 0 for no errors, 1 in case of any non-fatal error. */
+   Return value: 0 for no errors, 1 in case of any non-fatal error.
+   Note: If the return value is 0, no data must be sent to stdout. */
 int
 test_version (void)
 {
@@ -127,6 +146,7 @@ test_version (void)
   char buffer[256];
   int err = 0;
 
+#ifndef MPFR_USE_MINI_GMP
   sprintf (buffer, "%d.%d.%d", __GNU_MP_VERSION, __GNU_MP_VERSION_MINOR,
            __GNU_MP_VERSION_PATCHLEVEL);
   if (strcmp (buffer, gmp_version) != 0 &&
@@ -134,6 +154,7 @@ test_version (void)
        (sprintf (buffer, "%d.%d", __GNU_MP_VERSION, __GNU_MP_VERSION_MINOR),
         strcmp (buffer, gmp_version) != 0)))
     err = 1;
+#endif
 
   /* In some cases, it may be acceptable to have different versions for
      the header and the library, in particular when shared libraries are
@@ -153,7 +174,7 @@ test_version (void)
       printf ("  * A bad configuration in your include/library search paths.\n"
               "  * An inconsistency in the include/library search paths of\n"
               "    your development environment; an example:\n"
-              "      http://gcc.gnu.org/ml/gcc-help/2010-11/msg00359.html\n"
+              "      https://gcc.gnu.org/ml/gcc-help/2010-11/msg00359.html\n"
               "  * GMP has been upgraded after the first \"make check\".\n"
               "    In such a case, try again after a \"make clean\".\n"
               "  * A new or non-standard version naming is used in GMP.\n"
@@ -167,7 +188,6 @@ test_version (void)
   /* VL: I get the following error on an OpenSUSE machine, and changing
      the value of shlibpath_overrides_runpath in the libtool file from
      'no' to 'yes' fixes the problem. */
-
   version = mpfr_get_version ();
   if (strcmp (MPFR_VERSION_STRING, version) == 0)
     {
@@ -197,8 +217,15 @@ test_version (void)
       "  * On some platforms such as Solaris, $LD_LIBRARY_PATH overrides\n"
       "    the rpath, and if the MPFR library is already installed in a\n"
       "    $LD_LIBRARY_PATH directory, you typically get this error.  Do\n"
-      "    not use $LD_LIBRARY_PATH on such platforms; it may also break\n"
-      "    other things.\n"
+      "    not use $LD_LIBRARY_PATH permanently on such platforms; it may\n"
+      "    also break other things.\n"
+      "  * You may have an ld option that specifies a library search path\n"
+      "    where MPFR can be found, taking the precedence over the path\n"
+      "    added by libtool.  Check your environment variables, such as\n"
+      "    LD_OPTIONS under Solaris.  Moreover, under Solaris, the run path\n"
+      "    generated by libtool 2.4.6 may be incorrect: the build directory\n"
+      "    may not appear first in the run path; set $LD_LIBRARY_PATH to\n"
+      "    /path/to/builddir/src/.libs for the tests as a workaround.\n"
       "  * Then look at http://www.mpfr.org/mpfr-current/ for any update.\n"
       "  * Try again on a completely clean source (some errors might come\n"
       "    from a previous build or previous source changes).\n"
@@ -211,36 +238,63 @@ test_version (void)
   /* Note about $LD_LIBRARY_PATH under Solaris:
    *   https://en.wikipedia.org/wiki/Rpath#Solaris_ld.so
    * This cause has been confirmed by a user who got this error.
+   * And about the libtool 2.4.6 bug also concerning Solaris:
+   *   https://debbugs.gnu.org/cgi/bugreport.cgi?bug=30222
+   *   https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=888059
    */
   exit (1);
 }
 
+/* The inexact exception occurs very often, and is normal.
+   The underflow exception also might occur, for example in test_generic
+   for mpfr_xxx_d functions. Same for overflow. Thus we only check for
+   the division-by-zero and invalid exceptions, which should not occur
+   inside MPFR. */
+#define FPE_FLAGS (FE_DIVBYZERO | FE_INVALID)
+
 void
 tests_start_mpfr (void)
 {
-  test_version ();
-
-  /* don't buffer, so output is not lost if a test causes a segv etc */
+  /* Don't buffer, so output is not lost if a test causes a segv, etc.
+     For stdout, this is important as it will typically be fully buffered
+     by default with "make check". For stderr, the C standard just says
+     that it is not fully buffered (it may be line buffered by default);
+     disabling buffering completely might be useful in some cases.
+     Warning! No operations must have already been done on stdout/stderr
+     (this is a requirement of ISO C, and this is important on AIX).
+     Thus tests_start_mpfr should be called at the beginning of main(),
+     possibly after some variable settings. */
   setbuf (stdout, NULL);
+  setbuf (stderr, NULL);
+
+  test_version ();
 
 #if defined HAVE_LOCALE_H && defined HAVE_SETLOCALE
   /* Added on 2005-07-09. This allows to test MPFR under various
      locales. New bugs will probably be found, in particular with
      LC_ALL="tr_TR.ISO8859-9" because of the i/I character... */
-  setlocale (LC_ALL, "");
+  locale = setlocale (LC_ALL, "");
 #endif
 
 #ifdef MPFR_FPU_PREC
   set_fpu_prec ();
 #endif
 
-#ifdef MPFR_TEST_DIVBYZERO
+#ifdef MPFR_TESTS_FPE_DIV
   /* Define to test the use of MPFR_ERRDIVZERO */
   feclearexcept (FE_ALL_EXCEPT);
+# ifdef MPFR_TESTS_FPE_TRAP
+  /* to trap the corresponding FP exceptions */
+  feenableexcept (FPE_FLAGS);
+# endif
 #endif
 
-  tests_memory_start ();
-  tests_rand_start ();
+  if (tests_memory_disabled != 2)
+    {
+      if (tests_memory_disabled == 0)
+        tests_memory_start ();
+      tests_rand_start ();
+    }
   tests_limit_start ();
 
   default_emin = mpfr_get_emin ();
@@ -265,20 +319,28 @@ tests_end_mpfr (void)
     }
 
   mpfr_free_cache ();
-  tests_rand_end ();
-  tests_memory_end ();
-
-#ifdef MPFR_TEST_DIVBYZERO
-  /* Define to test the use of MPFR_ERRDIVZERO */
-  if (fetestexcept (FE_DIVBYZERO|FE_INVALID))
+  mpfr_free_cache2 (MPFR_FREE_GLOBAL_CACHE);
+  if (tests_memory_disabled != 2)
     {
-      printf ("A floating-point division by 0 or an invalid operation"
-              " occurred!\n");
-#ifdef MPFR_ERRDIVZERO
-      /* This should never occur because the purpose of defining
-         MPFR_ERRDIVZERO is to avoid all the FP divisions by 0. */
+      tests_rand_end ();
+      if (tests_memory_disabled == 0)
+        tests_memory_end ();
+    }
+
+#ifdef MPFR_TESTS_FPE_DIV
+  /* Define to test the use of MPFR_ERRDIVZERO */
+  if (fetestexcept (FPE_FLAGS))
+    {
+      /* With MPFR_ERRDIVZERO, such exceptions should never occur
+         because the purpose of defining MPFR_ERRDIVZERO is to avoid
+         all the FP divisions by 0. */
+      printf ("Some floating-point exception(s) occurred:");
+      if (fetestexcept (FE_DIVBYZERO))
+        printf (" DIVBYZERO");  /* e.g. from 1.0 / 0.0 to generate an inf */
+      if (fetestexcept (FE_INVALID))
+        printf (" INVALID");    /* e.g. from 0.0 / 0.0 to generate a NaN */
+      printf ("\n");
       err = 1;
-#endif
     }
 #endif
 
@@ -324,16 +386,16 @@ tests_rand_start (void)
   char           *perform_seed;
   unsigned long  seed;
 
-  if (__gmp_rands_initialized)
+  if (mpfr_rands_initialized)
     {
       printf (
-        "Please let tests_start() initialize the global __gmp_rands, i.e.\n"
+        "Please let tests_start() initialize the global mpfr_rands, i.e.\n"
         "ensure that function is called before the first use of RANDS.\n");
       exit (1);
     }
 
-  gmp_randinit_default (__gmp_rands);
-  __gmp_rands_initialized = 1;
+  gmp_randinit_default (mpfr_rands);
+  mpfr_rands_initialized = 1;
 
   perform_seed = getenv ("GMP_CHECK_RANDOMIZE");
   if (perform_seed != NULL)
@@ -342,7 +404,7 @@ tests_rand_start (void)
       if (! (seed == 0 || seed == 1))
         {
           printf ("Re-seeding with GMP_CHECK_RANDOMIZE=%lu\n", seed);
-          gmp_randseed_ui (__gmp_rands, seed);
+          gmp_randseed_ui (mpfr_rands, seed);
         }
       else
         {
@@ -355,13 +417,13 @@ tests_rand_start (void)
           time (&tv);
           seed = tv;
 #endif
-          gmp_randseed_ui (__gmp_rands, seed);
+          gmp_randseed_ui (mpfr_rands, seed);
           printf ("Seed GMP_CHECK_RANDOMIZE=%lu "
                   "(include this in bug reports)\n", seed);
         }
     }
   else
-    gmp_randseed_ui (__gmp_rands, 0x2143FEDC);
+    gmp_randseed_ui (mpfr_rands, 0x2143FEDC);
 }
 
 static void
@@ -526,7 +588,10 @@ ld_trace (const char *name, long double ld)
   printf ("] %.20Lg\n", ld);
 }
 
-/* Open a file in the src directory - can't use fopen directly */
+/* Open a file in the SRCDIR directory, i.e. the "tests" source directory,
+   which is different from the current directory when objdir is different
+   from srcdir. One should generally use this function instead of fopen
+   directly. */
 FILE *
 src_fopen (const char *filename, const char *mode)
 {
@@ -585,17 +650,17 @@ tests_default_random (mpfr_ptr x, int pos, mpfr_exp_t emin, mpfr_exp_t emax,
   MPFR_ASSERTN (emax <= MPFR_EMAX_MAX);
   /* but it isn't required that emin and emax are in the current
      exponent range (see below), so that underflow/overflow checks
-     can be done on 64-bit machines. */
+     can be done on 64-bit machines without a manual change of the
+     exponent range (well, this is a bit ugly...). */
 
   mpfr_urandomb (x, RANDS);
   if (MPFR_IS_PURE_FP (x) && (emin >= 1 || always_scale || (randlimb () & 1)))
     {
       mpfr_exp_t e;
-      e = MPFR_GET_EXP (x) +
-        (emin + (mpfr_exp_t) (randlimb () % (emax - emin + 1)));
+      e = emin + (mpfr_exp_t) (randlimb () % (emax - emin + 1));
       /* Note: There should be no overflow here because both terms are
-         between MPFR_EMIN_MIN and MPFR_EMAX_MAX, but the sum e isn't
-         necessarily between MPFR_EMIN_MIN and MPFR_EMAX_MAX. */
+         between MPFR_EMIN_MIN and MPFR_EMAX_MAX. */
+      MPFR_ASSERTD (e >= emin && e <= emax);
       if (mpfr_set_exp (x, e))
         {
           /* The random number doesn't fit in the current exponent range.
@@ -639,7 +704,7 @@ test5rm (int (*fct) (FLIST), mpfr_srcptr x, mpfr_ptr y, mpfr_ptr z,
 
       MPFR_ASSERTN (rnd != MPFR_RND_MAX);
       inex = fct (z, x, rnd);
-      if (! (mpfr_equal_p (y, z) || (mpfr_nan_p (y) && mpfr_nan_p (z))))
+      if (! SAME_VAL (y, z))
         {
           printf ("Error for %s with xprec=%lu, yprec=%lu, rnd=%s\nx = ",
                   name, (unsigned long) MPFR_PREC (x), (unsigned long) yprec,
@@ -795,8 +860,8 @@ data_check (const char *f, int (*foo) (FLIST), const char *name)
           ungetc (c, fp);
 
           c = fscanf (fp, "%ld %ld %c", &xprec, &yprec, &r);
-          MPFR_ASSERTN (xprec >= MPFR_PREC_MIN && xprec <= MPFR_PREC_MAX);
-          MPFR_ASSERTN (yprec >= MPFR_PREC_MIN && yprec <= MPFR_PREC_MAX);
+          MPFR_ASSERTN (MPFR_PREC_COND (xprec));
+          MPFR_ASSERTN (MPFR_PREC_COND (yprec));
           if (c == EOF)
             {
               perror ("data_check");
@@ -879,6 +944,11 @@ data_check (const char *f, int (*foo) (FLIST), const char *name)
  * mode for some lower precision: see data_check).
  * fct, inv, name: data related to the function.
  * pos, emin, emax: arguments for tests_default_random.
+ * For debugging purpose (e.g. in case of crash or infinite loop),
+ * you can set the MPFR_DEBUG_BADCASES environment variable to 1 in
+ * order to output information about the tested worst cases. You can
+ * also enable logging (when supported), but this may give too much
+ * information.
  */
 void
 bad_cases (int (*fct)(FLIST), int (*inv)(FLIST), const char *name,
@@ -1019,3 +1089,35 @@ flags_out (unsigned int flags)
     printf (" none");
   printf (" (%u)\n", flags);
 }
+
+static void
+abort_called (int x)
+{
+  /* Ok, abort has been called */
+  exit (0);
+}
+
+/* This function has to be called for a test
+   that will call the abort function */
+void
+tests_expect_abort (void)
+{
+#if defined(HAVE_SIGACTION)
+  struct sigaction act;
+  int ret;
+
+  memset (&act, 0, sizeof act);
+  act.sa_handler = abort_called;
+  ret = sigaction (SIGABRT, &act, NULL);
+  if (ret != 0)
+    {
+      /* Can't register error handler: Skip test */
+      exit (77);
+    }
+#elif defined(HAVE_SIGNAL)
+  signal (SIGABRT, abort_called);
+#else
+  /* Can't register error handler: Skip test */
+  exit (77);
+#endif
+}
